@@ -8,17 +8,15 @@ Created on Tue May 29 10:12:11 2018
 #try:
 #    from lxml import etree as ET
 #except ImportError:
-try:
-    import xml.etree.cElementTree as ET
-except ImportError:
-    import xml.etree.ElementTree as ET
-import requests
+import xml.etree.ElementTree as ET
 import pandas as pd
 import numpy as np
+import urllib.parse
+import urllib.request
 
 ### Parameters
 
-available_requests = ['SiteList', 'MeasurementList', 'GetData']
+available_requests = ['SiteList', 'MeasurementList', 'CollectionList', 'GetData']
 
 gauging_dict = {'Stage': {'row': 'I1', 'multiplier': 0.001},
                 'Flow [Gauging Results]': {'row': 'I2', 'multiplier': 0.001},
@@ -34,13 +32,14 @@ gauging_dict = {'Stage': {'row': 'I1', 'multiplier': 0.001},
                 'Stage Change [Gauging Results]': {'row': 'I12', 'multiplier': 1},
                 'Method': {'row': 'I13', 'multiplier': 1},
                 'Number Verts.': {'row': 'I14', 'multiplier': 1},
-                'Gauge Num.': {'row': 'I15', 'multiplier': 1}}
+                'Gauge Num.': {'row': 'I15', 'multiplier': 1},
+                'Stage [Gauging Results]': {'row': 'I1', 'multiplier': 0.001}}
 
 
 ### Functions
 
 
-def build_url(base_url, hts, request, site=None, measurement=None, from_date=None, to_date=None, location=None, site_parameters=None, agg_method=None, agg_interval=None, alignment=None, quality_codes=False, tstype=None):
+def build_url(base_url, hts, request, site=None, measurement=None, collection=None, from_date=None, to_date=None, location=None, site_parameters=None, agg_method=None, agg_interval=None, alignment=None, quality_codes=False, tstype=None):
     """
     Function to generate the Hilltop url for the web service.
 
@@ -56,6 +55,8 @@ def build_url(base_url, hts, request, site=None, measurement=None, from_date=Non
         The site to be extracted.
     measurement : str or None
         The measurement type name.
+    collection : str or None
+        The collection name.
     from_date : str or None
         The start date in the format 2001-01-01. None will put it to the beginning of the time series.
     to_date : str or None
@@ -84,63 +85,98 @@ def build_url(base_url, hts, request, site=None, measurement=None, from_date=Non
     ### Check base parameters
 
     if not base_url.endswith('/'):
-        base_url = base_url + '/'
+        base_url += '/'
     if not hts.endswith('.hts'):
         raise ValueError('The hts file must end with .hts')
     if request not in available_requests:
         raise ValueError('request must be one of ' + str(available_requests))
 
-    ### Make the first part of the url
-    url = base_url + hts + '?Service=Hilltop&Request=' + request
+    ### Collect data for a URL in a dict
+    data = {'Service': 'Hilltop', 'Request': request}
 
     ### Ready the others
-    if isinstance(site, (int, float, str)):
-        url = url + '&Site=' + requests.utils.quote(str(site))
-    if isinstance(measurement, str):
-        url = url + '&Measurement=' + requests.utils.quote(measurement)
-#    if isinstance(collection, str):
-#        url = url + '&Collection=' + requests.utils.quote(collection)
+    if site is not None:
+        data['Site'] = site
+    if measurement is not None:
+        data['Measurement'] = measurement
+    if collection is not None:
+        data['Collection'] = collection
     if isinstance(site_parameters, list):
-        url = url + '&SiteParameters=' + requests.utils.quote(','.join(site_parameters))
-    if isinstance(location, (str, bool)) and (request == 'SiteList'):
-        if isinstance(location, bool):
-            if location:
-                url = url + '&Location=Yes'
-        else:
-            url = url + '&Location=' + location
-    if quality_codes and (request == 'GetData'):
-        url = url + '&ShowQuality=Yes'
-
-    if tstype and (request == 'GetData'):
-        if tstype == 'Standard':
-            url = url + '&tsType=StdSeries'
-        elif tstype == 'Quality':
-            url = url + '&tsType=StdQualSeries'
-        elif tstype == 'Check':
-            url = url + '&tsType=CheckSeries'
-
-    ### Time interval goes last!
+        data['SiteParameters'] = ','.join(site_parameters)
+    if request == 'SiteList':
+        if location is True:
+            data['Location'] = 'Yes'
+        elif isinstance(location, str):
+            data['Location'] = location
     if request == 'GetData':
-        if isinstance(agg_method, str):
-            url = url + '&Method=' + requests.utils.quote(agg_method)
-        if isinstance(agg_interval, str):
-            url = url + '&Interval=' + requests.utils.quote(agg_interval)
-        if isinstance(from_date, str):
-            url = url + '&TimeInterval=' + from_date
-        else:
-            url = url + '&TimeInterval=1800-01-01'
-        if isinstance(to_date, str):
-            url = url + '/' + to_date
-        else:
-            url = url + '/now'
-        if isinstance(alignment, str):
-            url = url + '&Alignment=' + alignment
+        if quality_codes:
+            data['ShowQuality'] = 'Yes'
+        if tstype == 'Standard':
+            data['tsType'] = 'StdSeries'
+        elif tstype == 'Quality':
+            data['tsType'] = 'StdQualSeries'
+        elif tstype == 'Check':
+            data['tsType'] = 'CheckSeries'
+
+        ### Time interval goes last!
+        if agg_method is not None:
+            data['Method'] = agg_method
+        if agg_interval is not None:
+            data['Interval'] = agg_interval
+        if from_date is None:
+            from_date = '1800-01-01'
+        if to_date is None:
+            to_date = 'now'
+        data['TimeInterval'] = str(from_date) + '/' + str(to_date)
+        if alignment is not None:
+            data['Alignment'] = alignment
+
+    encoded_data = urllib.parse.urlencode(data, quote_via=urllib.parse.quote)
 
     ### return
-    return url
+    return base_url + hts + '?' + encoded_data
 
 
-def site_list(base_url, hts, location=None, measurement=None):
+def collection_list(base_url, hts):
+    """
+    CollectionList request function. Returns a frame of collection and site names associated with the hts file.
+
+    Parameters
+    ----------
+    base_url : str
+        root url str
+    hts : str
+        hts file name including the .hts extension.
+
+    Returns
+    -------
+    DataFrame
+    """
+    url = build_url(base_url, hts, 'CollectionList')
+    with urllib.request.urlopen(url) as req:
+        tree1 = ET.parse(req)
+    collection_tree = tree1.findall('Collection')
+    collection_list = []
+    for colitem in collection_tree:
+        colname = colitem.attrib['Name']
+        sitename_list = []
+        measurement_list = []
+        filename_list = []
+        for site in colitem:
+            row = dict([(col.tag, col.text) for col in site])
+            sitename_list.append(row['SiteName'])
+            measurement_list.append(row['Measurement'])
+            filename_list.append(row['Filename'])
+        col_df = pd.DataFrame({'SiteName': sitename_list})
+        col_df['Measurement'] = measurement_list
+        col_df['Filename'] = filename_list
+        col_df.insert(0, "CollectionName", colname)
+        collection_list.append(col_df)
+    collection_df = pd.concat(collection_list).reset_index(drop=True)
+    return collection_df
+
+
+def site_list(base_url, hts, location=None, measurement=None, collection=None):
     """
     SiteList request function. Returns a list of sites associated with the hts file.
 
@@ -152,14 +188,16 @@ def site_list(base_url, hts, location=None, measurement=None):
         hts file name including the .hts extension.
     location : str or bool
         Should the location be returned? Only applies to the SiteList request. 'Yes' returns the Easting and Northing, while 'LatLong' returns NZGD2000 lat lon coordinates.
+    collection : str
+        Get site list via a collection.
 
     Returns
     -------
     DataFrame
     """
-    url = build_url(base_url, hts, 'SiteList', location=location, measurement=measurement)
-    resp = requests.get(url, timeout=300)
-    tree1 = ET.fromstring(resp.content)
+    url = build_url(base_url, hts, 'SiteList', location=location, measurement=measurement, collection=collection)
+    with urllib.request.urlopen(url) as req:
+        tree1 = ET.parse(req)
     site_tree = tree1.findall('Site')
     if isinstance(location, (str, bool)):
         sites_dict = {}
@@ -224,8 +262,8 @@ def measurement_list(base_url, hts, site, measurement=None, output_bad_sites=Fal
     url = build_url(base_url, hts, 'MeasurementList', site, measurement)
 
     ### Request data and load in xml
-    resp = requests.get(url, timeout=300)
-    tree1 = ET.fromstring(resp.content)
+    with urllib.request.urlopen(url) as req:
+        tree1 = ET.parse(req)
     if tree1.find('Error') is not None:
         raise ValueError('No results returned from URL request')
     data_sources = tree1.findall('DataSource')
@@ -359,10 +397,9 @@ def get_data(base_url, hts, site, measurement, from_date=None, to_date=None, agg
     """
     ### Make url
     url = build_url(base_url=base_url, hts=hts, request='GetData', site=site, measurement=measurement, from_date=from_date, to_date=to_date, agg_method=agg_method, agg_interval=agg_interval, alignment=alignment, quality_codes=quality_codes, tstype=tstype)
-
     ### Request data and load in xml
-    resp = requests.get(url, timeout=300)
-    tree1 = ET.fromstring(resp.content)
+    with urllib.request.urlopen(url) as req:
+        tree1 = ET.parse(req)
     if tree1.find('Error') is not None:
         raise ValueError('No results returned from URL request')
     meas1 = tree1.find('Measurement')
@@ -538,8 +575,8 @@ def wq_sample_parameter_list(base_url, hts, site):
     url = build_url(base_url, hts, 'GetData', site, 'WQ Sample')
 
     ### Request data and load in xml
-    resp = requests.get(url, timeout=300)
-    tree1 = ET.fromstring(resp.content)
+    with urllib.request.urlopen(url) as req:
+        tree1 = ET.parse(req)
     if tree1.find('Error') is not None:
         raise ValueError('No results returned from URL request')
     meas1 = tree1.find('Measurement')
